﻿using System;


/// <summary>
/// מחלקה המגדירה ספר טלפונים המיוצג בעזרת מערך של אנשי קשר
/// </summary>
/// <author> צוות מדעי המחשב, המרכז להוראת המדעים, האוניברסיטה העברית, ירושלים </author>
/// <version> 28.11.2007 </version>
public class PhoneBook
{
    private Contact[] pbook;
    private int numOfContacts;

    public PhoneBook()
    {
        this.pbook = new Contact[100]; //גודל המערך שרירותי
        this.numOfContacts = 0;
    }

    public void AddContact(string name, string phone)
    {
        int index = getContactIndex(name);

        if (index != -1)
            this.pbook[index].SetPhone(phone);
        else
        {
            if (this.numOfContacts == this.pbook.Length) // אם התנאי מתקיים - מערך אנשי הקשר מלא ויש להגדילו 
            {
                Contact[] temp = new Contact[this.pbook.Length + 100]; //הגדלה המערך במספר שרירותי
                for (int i = 0; i < this.pbook.Length; i++)
                    temp[i] = this.pbook[i];
                this.pbook = temp;
            }
            this.pbook[this.numOfContacts] = new Contact(name, phone);
            this.numOfContacts++;
        }
    }

    public void DelContact(string name)
    {
        int index = getContactIndex(name);

        if (index != -1)
        {
            this.numOfContacts--;
            this.pbook[index] = this.pbook[this.numOfContacts];
            this.pbook[this.numOfContacts] = null;
        }
    }

    public string GetPhone(string name)
    {
        int index = getContactIndex(name);

        if (index != -1)
            return this.pbook[index].GetPhone();

        return null;
    }


    public string[] GetAllContactsNames()
    {
        // יצירת מערך בו ישמרו שמות אנשי הקשר
        string[] namesArr = new string[this.numOfContacts];

        for (int i = 0; i < this.numOfContacts; i++)
            namesArr[i] = this.pbook[i].GetName();

        return namesArr;
    }

    // פעולה זו ממינת את הרשימה בסדר אלפביתי על פי שם איש הקשר  
    // הפעולה משתמשת בפעולת עזר למיון מערך
    public override string ToString()
    {
        // מיון מערך השמות
        string[] namesArr = this.GetAllContactsNames();
        sortNames(namesArr);

        string str = "";
        for (int i = 0; i < namesArr.Length; i++)
            str = str + namesArr[i] + "\t" + this.GetPhone(namesArr[i]) + "\n";

        return str;
    }

    // פעולת עזר המחזירה את האינדקס בו נמצא איש הקשר עם השם המתקבל כפרמטר.
    // -1 אם לא קיים איש קשר בשם זה תחזיר הפעולה את הערך
    private int getContactIndex(string name)
    {
        for (int i = 0; i < this.numOfContacts; i++)
            if (this.pbook[i].GetName().Equals(name))
                return i;

        return -1;
    }

    // מיון מערך השמות בסדר עולה תוך שימוש באלגוריתם מיון בועות פשוט
    private void sortNames(string[] names)
    {
        string temp;

        for (int i = 1; i < names.Length; i++)
        {
            for (int j = 0; j < names.Length - i; j++)
            {
                if (names[j].CompareTo(names[j + 1]) > 0)
                {
                    temp = names[j];
                    names[j] = names[j + 1];
                    names[j + 1] = temp;
                }
            }
        }
    }
}
